@@ -32,12 +32,12 @@ Evaluate an end-to-end compression model on an image dataset.
 import argparse
 import json
 import math
-import os
 import sys
 import time
 
 from collections import defaultdict
-from typing import List
+from pathlib import Path
+from typing import Any, Dict, List
 
 import torch
 import torch.nn as nn
@@ -71,20 +71,29 @@ IMG_EXTENSIONS = (
 
 
 def collect_images(rootpath: str) -> List[str]:
-    return [
-        os.path.join(rootpath, f)
-        for f in os.listdir(rootpath)
-        if os.path.splitext(f)[-1].lower() in IMG_EXTENSIONS
-    ]
+    image_files = []
+    for ext in IMG_EXTENSIONS:
+        image_files.extend(Path(rootpath).glob(f"*{ext}"))
+    return image_files
 
 
-def psnr(a: torch.Tensor, b: torch.Tensor) -> float:
-    mse = F.mse_loss(a, b).item()
-    return -10 * math.log10(mse)
+def psnr(a: torch.Tensor, b: torch.Tensor, max_val: int = 255) -> float:
+    return 20 * math.log10(max_val) - 10 * torch.log10((a - b).pow(2).mean())
+
+
+def compute_metrics(
+    org: torch.Tensor, rec: torch.Tensor, max_val: int = 255
+) -> Dict[str, Any]:
+    metrics: Dict[str, Any] = {}
+    org = (org * max_val).clamp(0, max_val).round()
+    rec = (rec * max_val).clamp(0, max_val).round()
+    metrics["psnr"] = psnr(org, rec).item()
+    metrics["ms-ssim"] = ms_ssim(org, rec, data_range=max_val).item()
+    return metrics
 
 
 def read_image(filepath: str) -> torch.Tensor:
-    assert os.path.isfile(filepath)
+    assert filepath.is_file()
     img = Image.open(filepath).convert("RGB")
     return transforms.ToTensor()(img)
 
@@ -120,12 +129,14 @@ def inference(model, x):
         out_dec["x_hat"], (-padding_left, -padding_right, -padding_top, -padding_bottom)
     )
 
+    # input images are 8bit RGB for now
+    metrics = compute_metrics(x, out_dec["x_hat"], 255)
     num_pixels = x.size(0) * x.size(2) * x.size(3)
     bpp = sum(len(s[0]) for s in out_enc["strings"]) * 8.0 / num_pixels
 
     return {
-        "psnr": psnr(x, out_dec["x_hat"]),
-        "ms-ssim": ms_ssim(x, out_dec["x_hat"], data_range=1.0).item(),
+        "psnr": metrics["psnr"],
+        "ms-ssim": metrics["ms-ssim"],
         "bpp": bpp,
         "encoding_time": enc_time,
         "decoding_time": dec_time,
@@ -140,6 +151,8 @@ def inference_entropy_estimation(model, x):
     out_net = model.forward(x)
     elapsed_time = time.time() - start
 
+    # input images are 8bit RGB for now
+    metrics = compute_metrics(x, out_net["x_hat"], 255)
     num_pixels = x.size(0) * x.size(2) * x.size(3)
     bpp = sum(
         (torch.log(likelihoods).sum() / (-math.log(2) * num_pixels))
@@ -147,7 +160,8 @@ def inference_entropy_estimation(model, x):
     )
 
     return {
-        "psnr": psnr(x, out_net["x_hat"]),
+        "psnr": metrics["psnr"],
+        "ms-ssim": metrics["ms-ssim"],
         "bpp": bpp.item(),
         "encoding_time": elapsed_time / 2.0,  # broad estimation
         "decoding_time": elapsed_time / 2.0,
@@ -165,13 +179,22 @@ def load_checkpoint(arch: str, checkpoint_path: str) -> nn.Module:
     return architectures[arch].from_state_dict(state_dict).eval()
 
 
-def eval_model(model, filepaths, entropy_estimation=False, half=False):
+def eval_model(
+    model: nn.Module,
+    outputdir: Path,
+    filepaths,
+    entropy_estimation: bool = False,
+    trained_net: str = "",
+    description: str = "",
+    **args: Any,
+) -> Dict[str, Any]:
     device = next(model.parameters()).device
     metrics = defaultdict(float)
-    for f in filepaths:
-        x = read_image(f).to(device)
+    for filepath in filepaths:
+
+        x = read_image(filepath).to(device)
         if not entropy_estimation:
-            if half:
+            if args["half"]:
                 model = model.half()
                 x = x.half()
             rv = inference(model, x)
@@ -179,17 +202,30 @@ def eval_model(model, filepaths, entropy_estimation=False, half=False):
             rv = inference_entropy_estimation(model, x)
         for k, v in rv.items():
             metrics[k] += v
+        if args["per_image"]:
+            if Path(outputdir).is_dir():
+                image_metrics_path = (
+                    Path(outputdir) / f"{filepath.stem}-{trained_net}.json"
+                )
+                with image_metrics_path.open("wb") as f:
+                    output = {
+                        "source": filepath.stem,
+                        "name": args["architecture"],
+                        "description": f"Inference ({description})",
+                        "results": rv,
+                    }
+                    f.write(json.dumps(output, indent=2).encode())
+            else:
+                raise FileNotFoundError("Please specify output directory")
+
     for k, v in metrics.items():
         metrics[k] = v / len(filepaths)
     return metrics
 
 
 def setup_args():
-    parent_parser = argparse.ArgumentParser(
-        add_help=False,
-    )
-
     # Common options.
+    parent_parser = argparse.ArgumentParser(add_help=False)
     parent_parser.add_argument("dataset", type=str, help="dataset path")
     parent_parser.add_argument(
         "-a",
@@ -227,6 +263,33 @@ def setup_args():
         action="store_true",
         help="verbose mode",
     )
+    parent_parser.add_argument(
+        "-m",
+        "--metric",
+        type=str,
+        choices=["mse", "ms-ssim"],
+        default="mse",
+        help="metric trained against (default: %(default)s)",
+    )
+    parent_parser.add_argument(
+        "-d",
+        "--output_directory",
+        type=str,
+        default="",
+        help="path of output directory. Optional, required for output json file, results per image. Default will just print the output results.",
+    )
+    parent_parser.add_argument(
+        "-o",
+        "--output-file",
+        type=str,
+        default="",
+        help="output json file name, (default: architecture-entropy_coder.json)",
+    )
+    parent_parser.add_argument(
+        "--per-image",
+        action="store_true",
+        help="store results for each image of the dataset, separately",
+    )
 
     parser = argparse.ArgumentParser(
         description="Evaluate a model on an image dataset.", add_help=True
@@ -235,14 +298,6 @@ def setup_args():
 
     # Options for pretrained models
     pretrained_parser = subparsers.add_parser("pretrained", parents=[parent_parser])
-    pretrained_parser.add_argument(
-        "-m",
-        "--metric",
-        type=str,
-        choices=["mse", "ms-ssim"],
-        default="mse",
-        help="metric trained against (default: %(default)s)",
-    )
     pretrained_parser.add_argument(
         "-q",
         "--quality",
@@ -270,11 +325,15 @@ def main(argv):
     parser = setup_args()
     args = parser.parse_args(argv)
 
-    if not args.source:
+    if args.source not in ["checkpoint", "pretrained"]:
         print("Error: missing 'checkpoint' or 'pretrained' source.", file=sys.stderr)
         parser.print_help()
         raise SystemExit(1)
 
+    description = (
+        "entropy-estimation" if args.entropy_estimation else args.entropy_coder
+    )
+
     filepaths = collect_images(args.dataset)
     if len(filepaths) == 0:
         print("Error: no images found in directory.", file=sys.stderr)
@@ -282,12 +341,16 @@ def main(argv):
 
     compressai.set_entropy_coder(args.entropy_coder)
 
+    # create output directory
+    if args.output_directory:
+        Path(args.output_directory).mkdir(parents=True, exist_ok=True)
+
     if args.source == "pretrained":
         runs = sorted(args.qualities)
         opts = (args.architecture, args.metric)
         load_func = load_pretrained
         log_fmt = "\rEvaluating {0} | {run:d}"
-    elif args.source == "checkpoint":
+    else:
         runs = args.paths
         opts = (args.architecture,)
         load_func = load_checkpoint
@@ -299,9 +362,23 @@ def main(argv):
             sys.stderr.write(log_fmt.format(*opts, run=run))
             sys.stderr.flush()
         model = load_func(*opts, run)
+        if args.source == "pretrained":
+            trained_net = f"{args.architecture}-{args.metric}-{run}-{description}"
+        else:
+            cpt_name = Path(run).name[: -len(".tar.pth")]  # removesuffix() python3.9
+            trained_net = f"{cpt_name}-{description}"
+        print(f"Using trained model {trained_net}", file=sys.stderr)
         if args.cuda and torch.cuda.is_available():
             model = model.to("cuda")
-        metrics = eval_model(model, filepaths, args.entropy_estimation, args.half)
+        args_dict = vars(args)
+        metrics = eval_model(
+            model,
+            args.output_directory,
+            filepaths,
+            trained_net=trained_net,
+            description=description,
+            **args_dict,
+        )
         for k, v in metrics.items():
             results[k].append(v)
 
@@ -313,10 +390,21 @@ def main(argv):
         "entropy estimation" if args.entropy_estimation else args.entropy_coder
     )
     output = {
-        "name": args.architecture,
+        "name": f"{args.architecture}-{args.metric}",
         "description": f"Inference ({description})",
         "results": results,
     }
+    if args.output_directory:
+        output_file = (
+            args.output_file
+            if args.output_file
+            else f"{args.architecture}-{description}"
+        )
+
+        with (Path(f"{args.output_directory}/{output_file}").with_suffix(".json")).open(
+            "wb"
+        ) as f:
+            f.write(json.dumps(output, indent=2).encode())
 
     print(json.dumps(output, indent=2))
 
